#include "PcPort.h"

// Vordefinierte Pakete
static uint8_t okPacket[] = { 0xFF, 0x05, 0x0A, 0x00, 0xF0 };
static uint8_t badCrcPacket[] = { 0xFF, 0x05, 0x0A, 0x01, 0xF1 };
static uint8_t invalidPacket[] = { 0xFF, 0x05, 0x0A, 0x02, 0xF2 };
static uint8_t invalidI2CPins[] = { 0xFF, 0x05, 0x0A, 0x03, 0xF3 };
static uint8_t invalidPinType[] = { 0xFF, 0x05, 0x0A, 0x04, 0xF4 };
static uint8_t i2cInvalidAdress[] = { 0xFF, 0x05, 0x0A, 0x06, 0xF6 };
static uint8_t i2cIoError[] = { 0xFF, 0x05, 0x0A, 0x07, 0xF7 };
static uint8_t i2cBeginError[] = { 0xFF, 0x05, 0x0A, 0x08, 0xF8 };


// Wartet auf den Empfang eines Bytes.
static uint8_t readByte() {
   int b = Serial.read();
   while (b < 0)
      b = Serial.read();
   return b;
}

void PcPortClass::init()
{
   Serial.begin(9600);
   delay(100);
   while (Serial.available())
      Serial.read();
}

int PcPortClass::readPacket() {
   memset(packet.bytes, 0, sizeof(packet.bytes));
   uint8_t CRC = 0;
   uint8_t count = 0;

   int b = readByte();  // Startbyte
   while (b != 0xFF)
      b = readByte();

   packet.bytes[count++] = b;
   CRC = (byte)(CRC ^ b);


   uint8_t length = readByte(); // Lnge
   packet.bytes[count++] = length;
   CRC = (byte)(CRC ^ length);

   uint8_t paketType = readByte(); // Paket-Typ
   packet.bytes[count++] = paketType;
   CRC = (byte)(CRC ^ paketType);

   for (int i = 0; i < length - 4; i++)
   {
      b = readByte();
      packet.bytes[count++] = b;
      CRC = (byte)(CRC ^ b);
   }
   uint8_t crc = readByte();
   packet.bytes[count++] = crc;
   if (crc != CRC)
      return -1;

   return length;
}

void PcPortClass::sendResultPacket()
{
   uint8_t CRC = 0;

   //   DebugPrint("packet.count: "); DebugPrintln((int)packet.count);

   for (uint8_t i = 0; i < packet.count - 1; i++)
   {
      CRC ^= packet.bytes[i];
   }
   packet.bytes[packet.count - 1] = CRC;
   Serial.write(packet.bytes, packet.count);

   /*
   DebugPrintln("Sendpacket");
   DebugPrint("28..15: ");
   for (uint8_t i = 27; i >= 14; i--)
   {
      if (i == 23 || i == 19 || i == 15)
         DebugPrint(' ');
      DebugPrint((int)packet.data[i]);
   }
   DebugPrintln("");
   DebugPrint(" 1..14: ");
   for (uint8_t i = 0; i < 14; i++)
   {
      if (i == 4 || i == 8 || i == 12)
         DebugPrint(' ');
      DebugPrint((int)packet.data[i]);
   }
   DebugPrintln("");
   */
}

void PcPortClass::sendOK() { Serial.write(okPacket, sizeof(okPacket)); }

void PcPortClass::sendBadPacketType() { Serial.write(invalidPacket, sizeof(invalidPacket)); }

void PcPortClass::sendBadCrc() { Serial.write(badCrcPacket, sizeof(badCrcPacket)); }

void PcPortClass::sendBadI2CPins() { Serial.write(invalidI2CPins, sizeof(invalidI2CPins)); }

void PcPortClass::sendBadPinType() { Serial.write(invalidPinType, sizeof(invalidPinType)); }

void PcPortClass::sendI2cInvalidAdress() { Serial.write(i2cInvalidAdress, sizeof(i2cInvalidAdress)); }

void PcPortClass::sendI2cIoError() { Serial.write(i2cIoError, sizeof(i2cIoError)); }

void PcPortClass::sendi2cBeginError() { Serial.write(i2cBeginError, sizeof(i2cBeginError)); }

PcPortClass PcPort;