﻿namespace IC_DB
{
   /// <summary>
   /// Liste der durchzuführenden Testschritte.
   /// </summary>
   public class TestSteps : List<TestStep>
   {
      /// <summary>
      /// Initialisiert eine neue Instanz der <see cref="TestSteps"/>-Klasse.
      /// </summary>
      public TestSteps() { }

      /// <summary>
      /// Erstellt eine tiefe Kopie der Testschrittliste.
      /// </summary>
      /// <returns>Eine tiefe Kopie der Testschrittliste.</returns>
      public TestSteps DeepCopy()
      {
         TestSteps other = new TestSteps();
         for (int i = 0; i < Count; i++)
         {
            other.Add(this[i].DeepCopy());
         }
         return other;
      }

      /// <summary>
      /// Vergleicht, ob zwei Instanzen von <see cref="TestSteps"/> den gleichen Inhalt haben.
      /// </summary>
      /// <param name="obj1">Erstes Vergleichsobjekt.</param>
      /// <param name="obj2">Zweites Vergleichsobjekt.</param>
      /// <returns><b>true</b>, wenn die Inhalte beider Instanzen gleich sind, ansonsten <b>false</b>.</returns>
      public static bool operator ==(TestSteps obj1, TestSteps obj2)
      {
         if (obj1.Count != obj2.Count)
            return false;

         for (int i = 0; i < obj1.Count; i++)
            if (obj1[i] != obj2[i])
               return false;
         return true;
      }

      /// <summary>
      /// Vergleicht, ob zwei Instanzen von <see cref="TestStep"/> nicht den gleichen Inhalt haben.
      /// </summary>
      /// <param name="obj1">Erstes Vergleichsobjekt.</param>
      /// <param name="obj2">Zweites Vergleichsobjekt.</param>
      /// <returns><b>true</b>, wenn die Inhalte beider Instanzen nicht gleich sind, ansonsten <b>false</b>.</returns> 
      public static bool operator !=(TestSteps obj1, TestSteps obj2)
      { return !(obj1 == obj2); }


      /// <summary>
      /// Ermittelt, ob das angebene Onjekt identisch mit dem aktuellen ist.
      /// </summary>
      /// <param name="obj">Das zu vergleichende Objekt.</param>
      /// <returns><b>true</b>, wenn die Inhalte beider Instanzen gleich sind, ansonsten <b>false</b>.</returns>
      public override bool Equals(object? obj)
      {
         if (ReferenceEquals(this, obj))
         {
            return true;
         }

         if (ReferenceEquals(obj, null))
         {
            return false;
         }

         if (obj is TestSteps)
            return (TestSteps)obj == this;
         else
            return false;
      }

      /// <summary>
      /// Ermittelt den Hashcode für das aktuelle Objekt.
      /// </summary>
      /// <returns>Ein Hashcode für das aktuelle Objekt.</returns>
      public override int GetHashCode()
      {
         return base.GetHashCode();
      }
   }
}
