﻿using IC_DB;
using System.ComponentModel;
using IC_Tester_Controls;
using System.Reflection;
using System;

namespace IC_Tester
{
   //TODO: Das ist eine Mischung zwischen Rückmeldung und Testergegnissen. Besser wäre eine Trennung.
   /// <summary>
   /// Aufzählung der möglichen Testergebnisse.
   /// </summary>
   public enum TestResult
   {
      //TODO: Gründe besser beschreiben!

      //TODO: Werden alle Gründe benutzt?

      /// <summary>OK empfangen.</summary>
      [Description("OK")]
      [ToolTip("Fehlerfreie Ausführung des Testschritts.")]
      TestStepOk,

      /// <summary>Die Prüfsumme im empfangenen Paket ist falsch.</summary>
      /// <remarks>Fehler beim Empfang vom Arduino. RS232-Problem!</remarks>
      [Description("Ungültige CRC empfangen")]
      [ToolTip("Die CRC des Quittungspakets vom Arduino passt nicht. RS232-Problem.")]
      BadCRC,

      /// <summary>Arduino meldet: Paket mit falscher Prüfsumme empfangen.</summary>
      /// <remarks>Fehler beim Versand an Arduino. RS232-Problem!</remarks>
      [Description("Arduino: Ungültige CRC empfangen")]
      [ToolTip("Die CRC des gesendeten Testpakets an den Arduino passt nicht. RS232-Problem.")]
      ArduinoBadCRC,

      /// <summary>Arduino meldet: Unbekanter Pakettyp.</summary>
      /// <remarks>Programmierfehler.</remarks>
      [Description("Arduino: Ungültiger Paket-Typ empfangen")]
      [ToolTip("Das vom Arduino empfangene Testpakets hat eine ungültige Paketnummer. Programmierfehler!")]
      ArduinoBadPacketType, // Programmierfehler, sollte nicht auftreten.

      /// <summary>Arduino meldet: Ungültiger Pin-Typ.</summary>
      /// <remarks>Programmierfehler.</remarks>
      [Description("Arduino: Ungültiger Pin-Typ")]
      [ToolTip("Arduino: Im Datenpaket 'PinMode' Ungültiger Pin-Typ. Programmierfehler!")]
      ArduinoInvalidPinType, // Programmierfehler, sollte nicht auftreten.

      //TODO: ausführlicher beschreiben
      /// <summary>IO-Fehler beim Datenaustausch mit dem Arduino.</summary>
      [Description("IO-Fehler")]
      IoError,

      //TODO: ausführlicher beschreiben
      /// <summary>Testdatenpaket mit den Pin-Werten.</summary>
      [Description("Testdaten")]
      TestData,

      /// <summary>Rückmeldung der eingelesenen I2C-Daten.</summary>
      [Description("I2C-Testdaten")]
      [ToolTip("Rückmeldung der eingelesenen I2C-Daten.")]
      TestI2cData,

      //TODO: ausführlicher beschreiben
      /// <summary>Nicht die erwarteten Werte zurück geliefert.</summary>
      [Description("Test-Fehler")]
      TestStepError,

      //TODO: ausführlicher beschreiben
      /// <summary>Test wurde abgebrochen.</summary>
      [Description("Abbruch")]
      Abort,


      // I2C-Kommunikation mit einem I2C-Slave im Test
      // ----------------------------------------------------------------------------------

      /// <summary>Arduino meldet: Ungültige I2c-Pin-Angaben.</summary>
      /// <remarks>Fehler beim Festlegen der Pins in dem <see cref="IcDefinition"/>-Objekt.</remarks>
      [Description("Arduino: Ungültige I2c-Pin-Angaben")]
      [ToolTip("SDA und/oder SCL nicht als Pin hinterlegt. Pins kontrollieren!")]
      ArduinoI2CInvalidPins,

      /// <summary>Arduino meldet: I2C-Interface kann nicht initialisiert werden.</summary>
      /// <remarks>Die Kontrolle über SDA und/oder SCL kann nicht übernommen werden (twi.begin).</remarks>
      [Description("Arduino: I2C-Interface kann nicht initialisiert werden.")]
      [ToolTip("Die Kontrolle über SDA und/oder SCL kann nicht übernommen werden.\r\nI2C-Slave defekt?")]
      ArduinoI2CBeginError,

      /// <summary>Arduino meldet: Fehler bei der I2C-Kommunikation.</summary>
      /// <remarks>Es konnte nicht das komplette Datenpaket an den I2C-Slave übertragen werden.</remarks>
      [Description("Arduino: Fehler bei der I2C-Kommunikation.")]
      [ToolTip("Es konnte nicht das komplette Datenpaket an den I2C-Slave übertragen werden.\r\nI2C-Slave defekt?\r\nTestschritt falsch?")]
      ArduinoI2CIoError,

      /// <summary>Arduino meldet: Ungültige I2C Geräteadresse.</summary>
      /// <remarks>Kein I2C-Slave hat auf die gesendete Geräteadresse geantwortet.</remarks>
      [Description("Arduino: Ungültige I2C Geräteadresse.")]
      [ToolTip("Kein I2C-Slave hat auf die gesendete Geräteadresse geantwortet.\r\nI2C-Slave defekt?\r\nTestschritt falsch?")]
      ArduinoI2CInvalidAdress,


      /// <summary>Die eingelesenen I2C-Daten entprechen nicht der Erwartung.</summary>
      /// <remarks>Vorgaben im Testschritt und zurückgemeldete eingelesenen Daten sind ungleich.</remarks>
      [Description("Ungültige I2C Rückmeldung.")]
      [ToolTip("Die eingelesenen I2C-Daten entprechen nicht der Erwartung")]
      InvalidI2CReadData,

      // Bei weiteren Fehlern: Funktion isI2cError anpassen.
   }

   /// <summary>
   /// Erweiterungsmethoden für die <see cref="TestResult"/>-Klasse.
   /// </summary>
   public static class TestResultExtensions
   {
      /// <summary>
      /// Liest das <see cref="DescriptionAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DescriptionAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte Beschreibungstext oder der Name des Enumerationswerts.</returns>
      public static string GetDescription(this TestResult value)
      {
         return GetEnumDescription(value);
      }

      /// <summary>
      /// Liest das <see cref="DescriptionAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DescriptionAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte Beschreibungstext oder der Name des Enumerationswerts.</returns>
      public static string GetEnumDescription<T>(T value) where T : Enum
      {
         FieldInfo? fi = value.GetType().GetField(value.ToString());

         if (fi != null)
         {
            DescriptionAttribute[] attributes = (DescriptionAttribute[])fi.GetCustomAttributes(typeof(DescriptionAttribute), false);
            if (attributes != null && attributes.Length > 0)
            {
               return attributes[0].Description;
            }
         }
         return value.ToString();
      }

      /// <summary>
      /// Liest das <see cref="DisplayNameAttribute"/> des Enumerationswertes aus.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DisplayNameAttribute"/> ermittelt werden soll.</param>
      /// <returns>Der beim Enumerationswert hintelegte Beschreibungstext oder der Name des Enumerationswerts.</returns>
      public static string GetToolTipText(this TestResult value)
      {
         return ToolTipAttribute.GetToolTipText(value);
      }

      /// <summary>
      /// Bestimmt, ob ein Fehlercode ein Fehler beim Test det I²C-Schnittstelle ist.
      /// </summary>
      /// <param name="value">Der Enumerationswerte, dessen <see cref="DisplayNameAttribute"/> ermittelt werden soll.</param>
      /// <returns><b>true</b>, wenn der Fehlercode ein Fehler beim Test det I²C-Schnittstelle ist.</returns>
      public static bool isI2cError(this TestResult value)
      {
         return (value == TestResult.ArduinoI2CInvalidPins
             || value == TestResult.ArduinoI2CBeginError
             || value == TestResult.ArduinoI2CIoError
             || value == TestResult.ArduinoI2CInvalidAdress
             || value == TestResult.InvalidI2CReadData)
             ;
      }
   }
}